/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api.probes;

import org.ow2.petals.probes.api.exceptions.ProbeNotInitializedException;
import org.ow2.petals.probes.api.exceptions.ProbeNotStartedException;

/**
 * <p>
 * A gauge probe to probe instant value on demand, and storing the max reached value (that can be greater than the max
 * of instant values) since its last start.
 * </p>
 * <p>
 * <u><b>Lifecycle</b>:</u> The instant value can be acceded or picked only if the probe is started. The max value can
 * be acceded even if the probe is initialized or stopped. The max value is reseted to the sensor initial value when the
 * probe starts.
 * </p>
 * 
 * @param <T>
 *            The type of measured values (internal value type).
 * @param <S>
 *            The type of value to return to client probe (external value type).
 * 
 * @author Christophe DENEUX (PetalsLink)
 * 
 */
public interface GaugeProbe<T extends Comparable<T>, S> extends Probe {

    /**
     * <p>
     * Realizes a measure and compare the value to the max one to update it.
     * </p>
     * <p>
     * This method <b>MUST</b> be invoked by the application internals when the
     * resource associated to the probe changes.
     * </p>
     * <p>
     * This method is thread-safe
     * </p>
     * 
     * @throws ProbeNotStartedException
     *             The probe is not started.
     */
    public void pick() throws ProbeNotStartedException;

    /**
     * <p>
     * Accessor to the sensor instant value.
     * </p>
     * <p>
     * This method <b>MUST</b> be invoked by monitoring clients to get the probe
     * instant value.
     * </p>
     * <p>
     * This method is thread-safe
     * </p>
     * 
     * @return The sensor instant value.
     * @throws ProbeNotStartedException
     *             The probe is not started.
     */
    public S getInstantValue() throws ProbeNotStartedException;

    /**
     * <p>
     * Accessor to the max value.
     * </p>
     * <p>
     * This method is thread-safe
     * </p>
     * 
     * @return The probe max value.
     * @throws ProbeNotInitializedException
     *             The probe is not initialized.
     */
    public S getMaxValue() throws ProbeNotInitializedException;

}
