/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api.probes;

import org.ow2.petals.probes.api.exceptions.ProbeInitializationException;
import org.ow2.petals.probes.api.exceptions.ProbeInitializedException;
import org.ow2.petals.probes.api.exceptions.ProbeNotInitializedException;
import org.ow2.petals.probes.api.exceptions.ProbeNotStartedException;
import org.ow2.petals.probes.api.exceptions.ProbeShutdownException;
import org.ow2.petals.probes.api.exceptions.ProbeStartedException;
import org.ow2.petals.probes.api.exceptions.ProbeStartupException;
import org.ow2.petals.probes.api.exceptions.ProbeStopException;

/**
 * <p>
 * The probe interface, managing its lifecycle.
 * </p>
 * <p>
 * The probe lifecycle is:<br>
 * <code>
 * &nbsp;&nbsp;&nbsp;--> Initialized --> Started ---<br>
 * &nbsp;&nbsp;&nbsp;|&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|
 * &nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;|<br>
 * &nbsp;&nbsp;&nbsp;--- Shutdowned&nbsp; <-- Stopped <--
 * </code>
 * </p>
 * 
 * @author Christophe DENEUX (PetalsLink)
 * 
 */
public interface Probe {

    /**
     * Initializes the probe
     * 
     * @throws ProbeInitializedException
     *             The probe is already initialized, so can't be initialized
     *             until is is not shutdown.
     * @throws ProbeStartedException
     *             The probe is already started, so can't be initialized until
     *             it is not shutdown.
     * @throws ProbeInitializationException
     *             An error occurs during the probe initialization.
     */
    public void init() throws ProbeInitializedException, ProbeStartedException,
            ProbeInitializationException;

    /**
     * Starts the probe
     * 
     * @throws ProbeNotInitializedException
     *             The probe is not initialized, so can't be started until it is
     *             not initialized.
     * @throws ProbeStartedException
     *             The probe is already started, so can't be started until it is
     *             not just initialized.
     * @throws ProbeStartupException
     *             An error occurs during the probe startup.
     */
    public void start() throws ProbeNotInitializedException, ProbeStartedException,
            ProbeStartupException;

    /**
     * Stops the probe
     * 
     * @throws ProbeNotInitializedException
     *             The probe is not initialized, so can't be stopped until it is
     *             not initialized and started.
     * @throws ProbeNotStartedException
     *             The probe is not started, so can't be stopped until it is not
     *             started.
     * @throws ProbeStopException
     *             An error occurs during the probe stop.
     */
    public void stop() throws ProbeNotInitializedException, ProbeNotStartedException,
            ProbeStopException;

    /**
     * Shutdown the probe
     * 
     * @throws ProbeNotInitializedException
     *             The probe is not initialized.
     * @throws ProbeStartedException
     *             The probe is started.
     * @throws ProbeShutdownException
     *             An error occurs during the probe shutdown.
     */
    public void shutdown() throws ProbeShutdownException, ProbeStartedException,
            ProbeNotInitializedException;
}
