/**
 * Copyright (c) 2012 EBMWebsourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api.probes;

import java.util.Collections;
import java.util.LinkedList;

import org.ow2.petals.probes.api.exceptions.NoResponseTimeException;

/**
 * <p>
 * Relative response time values:
 * <ul>
 * <li>the max response time,</li>
 * <li>the avg response time,</li>
 * <li>the min response time.</li>
 * <li>the 10-percentile response time: 10% of the response times are lesser
 * than this value,</li>
 * <li>the 50-percentile response time: 50% of the response times are lesser
 * than this value,</li>
 * <li>the 90-percentile response time: 90% of the response times are upper than
 * this value.</li>
 * </ul>
 * </p>
 * <p>
 * For resources consumption, the avg response time is computed from the sum of
 * each response time and the number of response times, instead of keeping each
 * response time in memory.
 * </p>
 * 
 * @author Christophe DENEUX (PetalsLink)
 * 
 */
public class ResponseTimeRelativeValues {

    /**
     * The max response time
     */
    final private long max;

    /**
     * The sum of response times to computed the avg response time with
     * {@link #responseTimesNb}.
     */
    // TODO: What to do when the sum will exceed the max long value ?
    private long responseTimesSum = 0;

    /**
     * The number of response times to computed the avg response time with
     * {@link #responseTimesSum}.
     */
    // TODO: What to do when the sum will exceed the max long value ?
    final private long responseTimesNb;

    /**
     * The min response time
     */
    final private long min;

    /**
     * The 10-percentile response time
     */
    final private long percent10;

    /**
     * The 50-percentile response time
     */
    final private long percent50;

    /**
     * The 90-percentile response time
     */
    final private long percent90;

    /**
     * </p>Create response time values from a list of response times.</p>
     * 
     * @param responseTimes
     *            The list of response times from which the response time values
     *            will be computed.
     * @throws NoResponseTimeException
     *             No response time is provided (ie. <code>responseTimes</code>
     *             is empty).
     */
    public ResponseTimeRelativeValues(final LinkedList<Long> responseTimes)
            throws NoResponseTimeException {

        if (responseTimes.size() != 0) {
            Collections.sort(responseTimes);
            this.max = responseTimes.getLast();
            for (final Long responseTime : responseTimes) {
                this.responseTimesSum += responseTime;
            }
            this.min = responseTimes.getFirst();
            this.responseTimesNb = responseTimes.size();
            final long maxIndex = this.responseTimesNb - 1;
            this.percent10 = responseTimes.get((int) Math.round(maxIndex * 0.1));
            this.percent50 = responseTimes.get((int) Math.round(maxIndex * 0.5));
            this.percent90 = responseTimes.get((int) Math.round(maxIndex * 0.9));
        } else {
            throw new NoResponseTimeException();
        }
    }

    /**
     * @return The max response time
     */
    public long getMax() {
        return this.max;
    }

    /**
     * @return The avg response time
     */
    public long getAvg() {
        return this.responseTimesSum / this.responseTimesNb;
    }

    /**
     * @return The min response time
     */
    public long getMin() {
        return this.min;
    }

    /**
     * @return The 10-percentile response time
     */
    public long getPercent10() {
        return this.percent10;
    }

    /**
     * @return The 50-percentile response time
     */
    public long getPercent50() {
        return this.percent50;
    }

    /**
     * @return The 90-percentile response time
     */
    public long getPercent90() {
        return this.percent90;
    }

    /**
     * @return The sum of response times
     */
    public long getResponseTimesSum() {
        return responseTimesSum;
    }

    /**
     * @return The number of response times
     */
    public long getResponseTimesNb() {
        return this.responseTimesNb;
    }

}
