/**
 * Copyright (c) 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.probes.api;

import java.io.IOException;
import java.net.URL;
import java.util.Enumeration;

import org.junit.Test;
import org.ow2.petals.probes.api.exceptions.MultipleProbesFactoriesFoundException;
import org.ow2.petals.probes.api.exceptions.NoProbesFactoryFoundException;
import org.ow2.petals.probes.api.key.StringProbeKey;

import junit.framework.TestCase;

/**
 * Unit tests about keyed probe factory builder.
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class KeyedProbesFactoryBuilderTestCase {

    /**
     * <p>
     * The {@link KeyedProbesFactoryBuilder#getKeyedProbesFactory()} against
     * only one probe factory embedded into a JAR file of the classpath.
     * </p>
     * <p>
     * Expected results: no exception, the returned probe factory is the
     * expected one.
     * </p>
     */
    @Test
    public void testGetKeyedProbesFactory_001() {

        try {
            final KeyedProbesFactoryBuilder<StringProbeKey, Long, Long> probesFactoryBuilder = new KeyedProbesFactoryBuilder<StringProbeKey, Long, Long>();
            final KeyedProbesFactory<StringProbeKey, Long, Long> probesFactory = probesFactoryBuilder
                    .getKeyedProbesFactory();

            TestCase.assertEquals("The probes factory is not the expected one !",
                    TestKeyedProbesFactory.class, probesFactory.getClass());

        } catch (final MultipleProbesFactoriesFoundException e) {
            TestCase.fail(e.getMessage());
        } catch (final NoProbesFactoryFoundException e) {
            TestCase.fail(e.getMessage());
        }
    }

    /**
     * <p>
     * The {@link KeyedProbesFactoryBuilder#getKeyedProbesFactory(String)}
     * against a set of probe factories existing in the classpath.
     * </p>
     * <p>
     * Expected results: no exception, the returned probe factory is the
     * expected one.
     * </p>
     */
    @Test
    public void testGetKeyedProbesFactory_002() {

        try {
            final KeyedProbesFactoryBuilder<StringProbeKey, Long, Long> probesFactoryBuilder = new KeyedProbesFactoryBuilder<StringProbeKey, Long, Long>();
            final KeyedProbesFactory<StringProbeKey, Long, Long> probesFactory = probesFactoryBuilder
                    .getKeyedProbesFactory(TestKeyedProbesFactory.class.getName());

            TestCase.assertEquals("The probes factory is not the expected one !",
                    TestKeyedProbesFactory.class, probesFactory.getClass());

        } catch (final NoProbesFactoryFoundException e) {
            TestCase.fail(e.getMessage());
        }
    }

    /**
     * <p>
     * The {@link KeyedProbesFactoryBuilder#getKeyedProbesFactory()} against no
     * probe factory embedded into a JAR file of the classpath.
     * </p>
     * <p>
     * Expected results: the exception {@link NoProbesFactoryFoundException} is
     * thrown.
     * </p>
     */
    @Test
    public void testGetKeyedProbesFactory_101() {

        final ClassLoader classloaderWithoutSpi = new ClassLoader() {

            @Override
            public Enumeration<URL> getResources(String name) throws IOException {
                if ("META-INF/services/org.ow2.petals.probes.api.KeyedProbesFactory".equals(name)) {
                    // Use an inexisting resources to get an empty Enumeration.
                    return super.getResources("dummy-resources-that-does-not-exist");
                } else {
                    return super.getResources(name);
                }

            }
        };

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(classloaderWithoutSpi);

        try {
            final KeyedProbesFactoryBuilder<StringProbeKey, Long, Long> probesFactoryBuilder = new KeyedProbesFactoryBuilder<StringProbeKey, Long, Long>();
            final KeyedProbesFactory<StringProbeKey, Long, Long> probesFactory = probesFactoryBuilder
                    .getKeyedProbesFactory();

            TestCase.fail("The exception " + NoProbesFactoryFoundException.class.getSimpleName()
                    + " is not thrown !");

        } catch (final MultipleProbesFactoriesFoundException e) {
            TestCase.fail(e.getMessage());
        } catch (final NoProbesFactoryFoundException e) {
            // NOP, it's the expected exception
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * The {@link KeyedProbesFactoryBuilder#getKeyedProbesFactory()} against
     * several probe factories embedded into several JAR files of the classpath.
     * </p>
     * <p>
     * Expected results: the exception
     * {@link MultipleProbesFactoriesFoundException} is thrown.
     * </p>
     */
    @Test
    public void testGetKeyedProbesFactory_102() {

        final ClassLoader classloaderWithoutSpi = new ClassLoader() {

            @Override
            public Enumeration<URL> getResources(String name) throws IOException {
                if ("META-INF/services/org.ow2.petals.probes.api.KeyedProbesFactory".equals(name)) {
                    return super
                            .getResources("META-INF/services/org.ow2.petals.probes.api.MultipleKeyedProbesFactories");
                } else {
                    return super.getResources(name);
                }

            }
        };

        final ClassLoader oldClassloader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(classloaderWithoutSpi);

        try {
            final KeyedProbesFactoryBuilder<StringProbeKey, Long, Long> probesFactoryBuilder = new KeyedProbesFactoryBuilder<StringProbeKey, Long, Long>();
            final KeyedProbesFactory<StringProbeKey, Long, Long> probesFactory = probesFactoryBuilder
                    .getKeyedProbesFactory();

            TestCase.fail("The exception "
                    + MultipleProbesFactoriesFoundException.class.getSimpleName()
                    + " is not thrown !");

        } catch (final MultipleProbesFactoriesFoundException e) {
            // NOP, it's the expected exception
        } catch (final NoProbesFactoryFoundException e) {
            TestCase.fail(e.getMessage());
        } finally {
            Thread.currentThread().setContextClassLoader(oldClassloader);
        }
    }

    /**
     * <p>
     * The {@link KeyedProbesFactoryBuilder#getKeyedProbesFactory(String)}
     * against a probe factory not existing in the classpath.
     * </p>
     * <p>
     * Expected results: the exception {@link NoProbesFactoryFoundException} is
     * thrown.
     * </p>
     */
    @Test
    public void testGetKeyedProbesFactory_103() {

        try {
            final KeyedProbesFactoryBuilder<StringProbeKey, Long, Long> probesFactoryBuilder = new KeyedProbesFactoryBuilder<StringProbeKey, Long, Long>();
            final KeyedProbesFactory<StringProbeKey, Long, Long> probesFactory = probesFactoryBuilder
                    .getKeyedProbesFactory("org.ow2.petals.probes.api.NotExistingKeyedProbesFactory");

            TestCase.fail("The exception " + NoProbesFactoryFoundException.class.getSimpleName()
                    + " is not thrown !");

        } catch (final NoProbesFactoryFoundException e) {
            // NOP, it's the expected exception
        }
    }

}
